=== modified file 'Makefile'
--- Makefile	2009-09-07 19:09:58 +0000
+++ Makefile	2009-09-16 15:07:07 +0000
@@ -5,7 +5,7 @@
 RESULT="build/usb-creator-r$(REV)-$(RVER).exe"
 
 update-po:
-	find -regex "./\(usbcreator\|gui\|desktop\).*\.\(py\|glade\|ui\|in\)" \
+	find -regex "./\(usbcreator\|dbus\|gui\|desktop\).*\.\(py\|glade\|ui\|in\)" \
 		| sed '/\.ui/ s,^,[type: gettext/glade],' > po/POTFILES.in
 	echo main.py >> po/POTFILES.in
 	python setup.py build_i18n --merge-po --po-dir po

=== modified file 'bin/usb-creator-gtk'
--- bin/usb-creator-gtk	2009-09-01 17:25:39 +0000
+++ bin/usb-creator-gtk	2009-09-16 15:07:07 +0000
@@ -50,11 +50,6 @@
                   help=_('disable persistent setting by default in the UI'))
 (options, args) = parser.parse_args()
 
-if os.getuid() != 0:
-    args = ['gksu', 'gksu', '--desktop',
-    '/usr/share/applications/usb-creator-gtk.desktop', '--']
-    args.extend(sys.argv)
-    os.execvp(args[0], args)
 try:
     backend = DeviceKitBackend()
     frontend = GtkFrontend(backend, options.img, options.persistent)

=== added file 'bin/usb-creator-helper'
--- bin/usb-creator-helper	1970-01-01 00:00:00 +0000
+++ bin/usb-creator-helper	2009-09-16 15:07:07 +0000
@@ -0,0 +1,136 @@
+#!/usr/bin/python
+import dbus
+import gobject
+import dbus.service
+import logging
+from dbus.mainloop.glib import DBusGMainLoop
+from usbcreator.misc import *
+setup_logging()
+USBCREATOR_IFACE = 'com.ubuntu.USBCreator'
+PROPS_IFACE = 'org.freedesktop.DBus.Properties'
+
+class USBCreator(dbus.service.Object):
+    def __init__(self):
+        bus_name = dbus.service.BusName(USBCREATOR_IFACE, bus=dbus.SystemBus())
+        dbus.service.Object.__init__(self, bus_name, '/com/ubuntu/USBCreator')
+        self.dbus_info = None
+        self.polkit = None
+
+    # TODO return boolean success
+    @dbus.service.method(USBCREATOR_IFACE, in_signature='s', out_signature='',
+                         sender_keyword='sender', connection_keyword='conn')
+    def InstallBootloader(self, device, sender=None, conn=None):
+        '''Installs syslinux to the partition boot code area and writes the
+           syslinux boot code to the disk code area.  The latter is done to
+           handle cases where a bootloader is accidentally installed to the
+           MBR, and to handle some buggy BIOSes.
+
+           The function takes a partition device file of the form /dev/sda1.'''
+
+        self.check_polkit(sender, conn, 'com.ubuntu.usbcreator.bootloader')
+        bus = dbus.SystemBus()
+        device_file = device
+        devkit = bus.get_object('org.freedesktop.DeviceKit.Disks',
+                                   '/org/freedesktop/DeviceKit/Disks')
+        devkit = dbus.Interface(devkit, 'org.freedesktop.DeviceKit.Disks')
+        device = devkit.FindDeviceByDeviceFile(device)
+        deviceobj = bus.get_object('org.freedesktop.DeviceKit.Disks', device)
+        if deviceobj.Get(device, 'device-is-system-internal', dbus_interface=PROPS_IFACE):
+            raise dbus.DBusException, 'com.ubuntu.USBCreator.Error.SystemInternal'
+        
+        popen(['syslinux', '-f', device_file])
+        # Find the parent of the partition.
+        parent = deviceobj.Get(device, 'partition-slave',
+                               dbus_interface=PROPS_IFACE)
+        parentobj = bus.get_object('org.freedesktop.DeviceKit.Disks', parent)
+        parent = parentobj.Get(parent, 'device-file',
+                               dbus_interface=PROPS_IFACE)
+        # Write the syslinux MBR.
+        popen(['dd', 'if=/usr/lib/syslinux/mbr.bin', 'of=%s' % parent,
+               'bs=446', 'count=1', 'conv=sync'])
+        num = deviceobj.Get(device, 'partition-number',
+                            dbus_interface=PROPS_IFACE)
+        popen(['/sbin/parted', parent, 'set', str(num), 'boot', 'on'])
+
+    @dbus.service.method(USBCREATOR_IFACE, in_signature='s', out_signature='',
+                         sender_keyword='sender', connection_keyword='conn')
+    def Format(self, device, sender=None, conn=None):
+        self.check_polkit(sender, conn, 'com.ubuntu.usbcreator.format')
+    
+    @dbus.service.method(USBCREATOR_IFACE, in_signature='ss', out_signature='',
+                         sender_keyword='sender', connection_keyword='conn')
+    def Image(self, source, target, sender=None, conn=None):
+        self.check_polkit(sender, conn, 'com.ubuntu.usbcreator.image')
+        cmd = ['dd', 'if=%s' % str(source), 'of=%s' % str(target), 'bs=1M']
+        popen(cmd)
+    
+    @dbus.service.method(USBCREATOR_IFACE, in_signature='s', out_signature='s',
+                         sender_keyword='sender', connection_keyword='conn')
+    def Mount(self, device, sender=None, conn=None):
+        self.check_polkit(sender, conn, 'com.ubuntu.usbcreator.mount')
+        import tempfile
+        ret = tempfile.mkdtemp()
+        popen(['mount', '-o', 'loop', device, ret])
+        return ret
+    
+    @dbus.service.method(USBCREATOR_IFACE, in_signature='s', out_signature='',
+                         sender_keyword='sender', connection_keyword='conn')
+    def Unmount(self, device, sender=None, conn=None):
+        self.check_polkit(sender, conn, 'com.ubuntu.usbcreator.mount')
+        try:
+            popen(['umount', device])
+        except USBCreatorProcessException:
+            logging.exception('Could not unmount %s:' % device)
+    
+    @dbus.service.method(USBCREATOR_IFACE, in_signature='s', out_signature='',
+                         sender_keyword='sender', connection_keyword='conn')
+    def RemountRW(self, device, sender=None, conn=None):
+        self.check_polkit(sender, conn, 'com.ubuntu.usbcreator.mount')
+        popen(['mount', '-o', 'remount,rw', device])
+    
+    @dbus.service.method(USBCREATOR_IFACE, in_signature='', out_signature='',
+                         sender_keyword='sender', connection_keyword='conn')
+    def Shutdown(self, sender=None, conn=None):
+        logging.debug('Shutting down.')
+        loop.quit()
+
+    # Taken from Jockey 0.5.3.
+    def check_polkit(self, sender, conn, priv):
+        if sender is None and conn is None:
+            return
+        if self.dbus_info is None:
+            self.dbus_info = dbus.Interface(conn.get_object(
+                                            'org.freedesktop.DBus',
+                                            '/org/freedesktop/DBus/Bus',
+                                            False), 'org.freedesktop.DBus')
+        pid = self.dbus_info.GetConnectionUnixProcessID(sender)
+        if self.polkit is None:
+            self.polkit = dbus.Interface(dbus.SystemBus().get_object(
+                                'org.freedesktop.PolicyKit1',
+                                '/org/freedesktop/PolicyKit1/Authority',
+                                False), 'org.freedesktop.PolicyKit1.Authority')
+        try:
+            # we don't need is_challenge return here, since we call with
+            # AllowUserInteraction
+            (is_auth, _, details) = self.polkit.CheckAuthorization(
+                                    ('unix-process', {'pid': dbus.UInt32(pid,
+                                    variant_level=1)}), priv, {'': ''},
+                                    dbus.UInt32(1), '', timeout=600)
+        except dbus.DBusException, e:
+            if e._dbus_error_name == 'org.freedesktop.DBus.Error.ServiceUnknown':
+                # polkitd timed out, connect again
+                self.polkit = None
+                return self.check_polkit(sender, conn, priv)
+            else:
+                raise
+
+        if not is_auth:
+            logging.debug('_check_polkit_privilege: sender %s on connection %s '
+                          'pid %i is not authorized for %s: %s' %
+                          (sender, conn, pid, priv, str(details)))
+            raise dbus.DBusException, 'com.ubuntu.USBCreator.Error.NotAuthorized'
+
+DBusGMainLoop(set_as_default=True)
+helper = USBCreator()
+loop = gobject.MainLoop()
+loop.run()

=== modified file 'bin/usb-creator-kde'
--- bin/usb-creator-kde	2009-09-01 06:19:23 +0000
+++ bin/usb-creator-kde	2009-09-16 15:07:07 +0000
@@ -66,11 +66,6 @@
     if args.isSet("persistent"):
         persistent = True
     
-    if os.getuid() != 0:
-        args = ['kdesudo', 'usb-creator-kde']
-        args.extend(sys.argv)
-        os.execvp(args[0], args)
-
     try:
         backend = DeviceKitBackend()
         frontend = KdeFrontend(backend, img, persistent)

=== added directory 'dbus'
=== added file 'dbus/com.ubuntu.USBCreator.conf'
--- dbus/com.ubuntu.USBCreator.conf	1970-01-01 00:00:00 +0000
+++ dbus/com.ubuntu.USBCreator.conf	2009-09-16 15:07:07 +0000
@@ -0,0 +1,23 @@
+<!DOCTYPE busconfig PUBLIC
+ "-//freedesktop//DTD D-BUS Bus Configuration 1.0//EN"
+ "http://www.freedesktop.org/standards/dbus/1.0/busconfig.dtd">
+<busconfig>
+
+  <!-- Only root can own the service -->
+  <policy user="root">
+    <allow own="com.ubuntu.USBCreator"/>
+  </policy>
+
+  <!-- Allow anyone to invoke methods (further constrained by
+       PolicyKit privileges -->
+  <policy context="default">
+    <allow send_destination="com.ubuntu.USBCreator" 
+           send_interface="com.ubuntu.USBCreator"/>
+    <allow send_destination="com.ubuntu.USBCreator" 
+           send_interface="org.freedesktop.DBus.Introspectable"/>
+    <allow send_destination="com.ubuntu.USBCreator" 
+           send_interface="org.freedesktop.DBus.Properties"/>
+  </policy>
+
+</busconfig>
+

=== added file 'dbus/com.ubuntu.USBCreator.service'
--- dbus/com.ubuntu.USBCreator.service	1970-01-01 00:00:00 +0000
+++ dbus/com.ubuntu.USBCreator.service	2009-09-16 15:07:07 +0000
@@ -0,0 +1,4 @@
+[D-BUS Service]
+Name=com.ubuntu.USBCreator
+Exec=/usr/bin/usb-creator-helper
+User=root

=== added file 'dbus/com.ubuntu.usbcreator.policy.in'
--- dbus/com.ubuntu.usbcreator.policy.in	1970-01-01 00:00:00 +0000
+++ dbus/com.ubuntu.usbcreator.policy.in	2009-09-16 15:07:07 +0000
@@ -0,0 +1,48 @@
+<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE policyconfig PUBLIC
+ "-//freedesktop//DTD PolicyKit Policy Configuration 1.0//EN"
+ "http://www.freedesktop.org/standards/PolicyKit/1.0/policyconfig.dtd">
+<policyconfig>
+
+  <vendor>USB Creator</vendor>
+  <vendor_url>https://launchpad.net/usb-creator</vendor_url>
+  <icon_name>usb-creator-gtk</icon_name>
+
+  <action id="com.ubuntu.usbcreator.bootloader">
+    <_description>Install the bootloader</_description>
+    <_message>System policy prevents installing the bootloader</_message>
+    <defaults>
+      <allow_any>no</allow_any>
+      <allow_inactive>no</allow_inactive>
+      <allow_active>yes</allow_active>
+    </defaults>
+  </action>
+  <action id="com.ubuntu.usbcreator.format">
+    <_description>Format the device</_description>
+    <_message>System policy prevents formatting this device</_message>
+    <defaults>
+      <allow_any>no</allow_any>
+      <allow_inactive>no</allow_inactive>
+      <allow_active>yes</allow_active>
+    </defaults>
+  </action>
+  <action id="com.ubuntu.usbcreator.image">
+    <_description>Image the device</_description>
+    <_message>System policy prevents writing a disk image to this device</_message>
+    <defaults>
+      <allow_any>no</allow_any>
+      <allow_inactive>no</allow_inactive>
+      <allow_active>yes</allow_active>
+    </defaults>
+  </action>
+  <action id="com.ubuntu.usbcreator.mount">
+    <_description>Mount a device</_description>
+    <_message>System policy prevents mounting</_message>
+    <defaults>
+      <allow_any>no</allow_any>
+      <allow_inactive>no</allow_inactive>
+      <allow_active>yes</allow_active>
+    </defaults>
+  </action>
+</policyconfig> 
+

=== modified file 'debian/usb-creator-common.install'
--- debian/usb-creator-common.install	2009-08-18 12:39:38 +0000
+++ debian/usb-creator-common.install	2009-09-16 15:07:07 +0000
@@ -4,3 +4,7 @@
 usr/lib/python*/*/usbcreator/frontends/*.py
 usr/lib/python*/*/usbcreator/frontends/base/*
 usr/lib/python*/*/usbcreator/*.py
+usr/bin/usb-creator-helper
+usr/share/polkit-1/actions/*
+usr/share/dbus-1/system-services/*
+etc/dbus-1/system.d/*

=== modified file 'po/usbcreator.pot'
--- po/usbcreator.pot	2009-09-08 14:34:51 +0000
+++ po/usbcreator.pot	2009-09-16 15:07:07 +0000
@@ -8,7 +8,7 @@
 msgstr ""
 "Project-Id-Version: PACKAGE VERSION\n"
 "Report-Msgid-Bugs-To: \n"
-"POT-Creation-Date: 2009-09-08 15:32+0100\n"
+"POT-Creation-Date: 2009-09-16 15:53+0100\n"
 "PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
 "Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
 "Language-Team: LANGUAGE <LL@li.org>\n"
@@ -35,7 +35,7 @@
 msgstr ""
 
 #: .././gui/usbcreator-gtk.ui.h:3
-#: .././usbcreator/frontends/kde/frontend.py:439
+#: .././usbcreator/frontends/kde/frontend.py:442
 msgid "Do you really want to quit the installation now?"
 msgstr ""
 
@@ -72,7 +72,7 @@
 msgstr ""
 
 #: .././gui/usbcreator-gtk.ui.h:11
-#: .././usbcreator/frontends/kde/frontend.py:438
+#: .././usbcreator/frontends/kde/frontend.py:441
 msgid "Quit the installation?"
 msgstr ""
 
@@ -139,52 +139,52 @@
 #. base frontend.
 #. To be displayed as a list of file type filters.
 #: .././usbcreator/frontends/gtk/frontend.py:442
-#: .././usbcreator/frontends/kde/frontend.py:372
+#: .././usbcreator/frontends/kde/frontend.py:374
 #: .././usbcreator/frontends/winui/frontend.py:177
 msgid "CD Images"
 msgstr ""
 
 #: .././usbcreator/frontends/gtk/frontend.py:442
-#: .././usbcreator/frontends/kde/frontend.py:372
+#: .././usbcreator/frontends/kde/frontend.py:374
 #: .././usbcreator/frontends/winui/frontend.py:178
 msgid "Disk Images"
 msgstr ""
 
 #. TODO evand 2009-07-31: Make these the default values in the
 #. GtkBuilder file.
-#: .././usbcreator/frontends/gtk/frontend.py:466
+#: .././usbcreator/frontends/gtk/frontend.py:468
 #: .././usbcreator/frontends/winui/install_window.py:28
 msgid "Starting up..."
 msgstr ""
 
-#: .././usbcreator/frontends/gtk/frontend.py:486
+#: .././usbcreator/frontends/gtk/frontend.py:488
 #: .././usbcreator/frontends/winui/frontend.py:151
 #, python-format
 msgid "%d%% complete (%dm%ss remaining)"
 msgstr ""
 
-#: .././usbcreator/frontends/gtk/frontend.py:490
+#: .././usbcreator/frontends/gtk/frontend.py:492
 #: .././usbcreator/frontends/winui/frontend.py:155
 #, python-format
 msgid "%d%% complete"
 msgstr ""
 
 #. TODO evand 2009-07-28: Better error message.
-#: .././usbcreator/frontends/gtk/frontend.py:515
-#: .././usbcreator/frontends/kde/frontend.py:195
+#: .././usbcreator/frontends/gtk/frontend.py:517
+#: .././usbcreator/frontends/kde/frontend.py:197
 #: .././usbcreator/frontends/winui/frontend.py:85
 msgid "Installation failed."
 msgstr ""
 
-#: .././usbcreator/frontends/kde/frontend.py:387
+#: .././usbcreator/frontends/kde/frontend.py:390
 msgid "Starting up"
 msgstr ""
 
-#: .././usbcreator/frontends/kde/frontend.py:395
+#: .././usbcreator/frontends/kde/frontend.py:398
 msgid "You must select both source image and target device first."
 msgstr ""
 
-#: .././usbcreator/frontends/kde/frontend.py:421
+#: .././usbcreator/frontends/kde/frontend.py:424
 msgid ""
 "Installation is complete.  You may now reboot your computer with this USB "
 "thumb drive inserted to boot Ubuntu."
@@ -271,42 +271,38 @@
 "(%d MB) > %s (%d MB)"
 msgstr ""
 
-#. TODO evand 2009-07-23: Zero out the MBR.  Check to see if the first
-#. 446 bytes are all NULs, and if not, ask the user if they want to wipe
-#. it.  Confirm with a USB disk that never has had an OS installed to
-#. it.
-#: .././usbcreator/install.py:191
+#: .././usbcreator/install.py:187
 msgid "Installing the bootloader..."
 msgstr ""
 
-#: .././usbcreator/install.py:203
+#: .././usbcreator/install.py:188
 msgid "Failed to install the bootloader."
 msgstr ""
 
-#: .././usbcreator/install.py:211
+#: .././usbcreator/install.py:215
 msgid "Modifying configuration..."
 msgstr ""
 
-#: .././usbcreator/install.py:278
+#: .././usbcreator/install.py:282
 msgid "Creating a persistence file..."
 msgstr ""
 
-#: .././usbcreator/install.py:281
+#: .././usbcreator/install.py:285
 msgid "Creating an ext2 filesystem in the persistence file..."
 msgstr ""
 
-#: .././usbcreator/install.py:292
+#: .././usbcreator/install.py:296
 msgid "Flushing writes to the disk..."
 msgstr ""
 
 #. TODO evand 2009-09-02: Disabled until we can find a cross-platform
 #. way of determining dd progress.
 #. self.initialize_progress_thread()
-#: .././usbcreator/install.py:301
+#: .././usbcreator/install.py:305
 msgid "Writing disk image..."
 msgstr ""
 
-#: .././usbcreator/install.py:302
+#: .././usbcreator/install.py:306
 #, python-format
 msgid "Could not write the disk image (%s) to the device (%s)."
 msgstr ""
@@ -314,15 +310,15 @@
 #. Clear.
 #. Some of the code in this function was copied from Ubiquity's
 #. scripts/install.py
-#: .././usbcreator/install.py:332 .././usbcreator/install.py:415
+#: .././usbcreator/install.py:347 .././usbcreator/install.py:430
 msgid "Removing files..."
 msgstr ""
 
-#: .././usbcreator/install.py:364 .././usbcreator/install.py:439
+#: .././usbcreator/install.py:379 .././usbcreator/install.py:454
 msgid "Copying files..."
 msgstr ""
 
-#: .././usbcreator/install.py:393
+#: .././usbcreator/install.py:408
 msgid "md5 checksums do not match."
 msgstr ""
 
@@ -331,16 +327,16 @@
 #. user the option of selecting the re-inserted disk
 #. from a drop down list and continuing.
 #. TODO evand 2009-07-23: Fail more gracefully.
-#: .././usbcreator/install.py:503
+#: .././usbcreator/install.py:518
 #, python-format
 msgid "Could not read from %s"
 msgstr ""
 
-#: .././usbcreator/install.py:528
+#: .././usbcreator/install.py:543
 msgid "Checksums do not match.  Retry?"
 msgstr ""
 
-#: .././usbcreator/install.py:532
+#: .././usbcreator/install.py:547
 msgid "Checksums do not match."
 msgstr ""
 

=== modified file 'setup.cfg'
--- setup.cfg	2009-06-26 22:06:31 +0000
+++ setup.cfg	2009-09-16 15:07:07 +0000
@@ -3,4 +3,4 @@
 
 [build_i18n]
 domain = usbcreator
-desktop_files = [('share/applications', ('desktop/usb-creator-gtk.desktop.in','desktop/usb-creator-kde.desktop.in',))]
+desktop_files = [('share/applications', ['desktop/usb-creator-gtk.desktop.in', 'desktop/usb-creator-kde.desktop.in']), ('share/polkit-1/actions', ['dbus/com.ubuntu.usbcreator.policy.in'])]

=== modified file 'setup.py'
--- setup.py	2009-09-01 17:25:39 +0000
+++ setup.py	2009-09-16 15:07:07 +0000
@@ -16,10 +16,12 @@
               'usbcreator.backends.base',
               'usbcreator.backends.devicekit',
              ],
-    scripts=['bin/usb-creator-gtk','bin/usb-creator-kde'],
+    scripts=['bin/usb-creator-gtk','bin/usb-creator-kde', 'bin/usb-creator-helper'],
     data_files=[('share/usb-creator', ['gui/usbcreator-gtk.ui']),
                 ('share/pixmaps', ['desktop/usb-creator-gtk.png', 'desktop/usb-creator-kde.png']),
-                ('share/kde4/apps/usb-creator-kde', ['gui/usbcreator-kde.ui'])],
+                ('share/kde4/apps/usb-creator-kde', ['gui/usbcreator-kde.ui']),
+                ('/etc/dbus-1/system.d', ['dbus/com.ubuntu.USBCreator.conf']),
+                ('share/dbus-1/system-services', ['dbus/com.ubuntu.USBCreator.service'])],
     cmdclass = { "build" : build_extra.build_extra,
         "build_i18n" :  build_i18n.build_i18n,
         "build_help" :  build_help.build_help,

=== modified file 'usbcreator/backends/devicekit/backend.py'
--- usbcreator/backends/devicekit/backend.py	2009-09-08 15:22:01 +0000
+++ usbcreator/backends/devicekit/backend.py	2009-09-16 15:07:07 +0000
@@ -23,6 +23,9 @@
                                          '/org/freedesktop/DeviceKit/Disks')
         self.devkit = dbus.Interface(devkit_obj,
                                      'org.freedesktop.DeviceKit.Disks')
+        self.helper = self.bus.get_object('com.ubuntu.USBCreator',
+                                          '/com/ubuntu/USBCreator')
+        self.helper = dbus.Interface(self.helper, 'com.ubuntu.USBCreator')
 
     # Device detection and processing functions.
 
@@ -239,11 +242,9 @@
             else:
                 source = get('device-mount-paths')[0]
         elif stype == SOURCE_ISO:
-            import tempfile
             isofile = self.sources[source]['device']
-            source = tempfile.mkdtemp()
+            source = self.helper.Mount(isofile)
             self.mounted_source = source
-            popen(['mount', '-o', 'loop', isofile, source])
         
         dk = self.bus.get_object('org.freedesktop.DeviceKit.Disks', target)
         def get(prop):
@@ -257,24 +258,20 @@
                 target = dk.FilesystemMount('', [], dbus_interface=DEVICE_IFACE)
             else:
                 target = get('device-mount-paths')[0]
-            popen(['mount', '-o', 'remount,rw', dev])
-            try:
-                parent = get('partition-slave')
-                d = self.bus.get_object(DISKS_IFACE, parent)
-                parent = d.Get(parent, 'device-file', dbus_interface=PROPS_IFACE)
-                popen(['dd', 'if=/usr/lib/syslinux/mbr.bin', 'of=%s' % parent,
-                       'bs=446', 'count=1', 'conv=sync'])
-                try:
-                    num = get('partition-number')
-                    popen(['parted', parent, 'set', str(num), 'boot', 'on'])
-                except:
-                    logging.exception('Could not set the boot flag.')
-            except:
-                logging.exception('Could not clear the boot sector code area.')
-
+            # XXX evand 2009-09-16: In my opinion, it should be possible to do
+            # this:
+            # dk.FilesystemMount('', ['remount,rw'], dbus_interface=DEVICE_IFACE)
+            # 
+            # But that returns "org.freedesktop.DeviceKit.Disks.Error.Busy:
+            # /dev/sdb1 is mounted", so we'll have to work around it for now.
+            self.helper.RemountRW(dev)
         Backend.install(self, source, target, persist, device=dev)
 
     def cancel_install(self):
         Backend.cancel_install(self)
         if self.mounted_source:
-            popen(['umount', self.mounted_source])
+            self.helper.Unmount(self.mounted_source)
+        try:
+            self.helper.Shutdown()
+        except dbus.DBusException:
+            logging.exception('Could not shut down the dbus service.')

=== modified file 'usbcreator/install.py'
--- usbcreator/install.py	2009-09-08 14:21:10 +0000
+++ usbcreator/install.py	2009-09-16 15:07:07 +0000
@@ -17,7 +17,7 @@
 import stat
 import sys
 import shutil
-from usbcreator.misc import popen, USBCreatorProcessException, fs_size
+from usbcreator.misc import popen, USBCreatorProcessException, fs_size, MAX_DBUS_TIMEOUT
 from usbcreator.remtimest import RemainingTimeEstimator
 from threading import Thread, Event
 import logging
@@ -184,26 +184,30 @@
 
     def install_bootloader(self):
         logging.debug('install_bootloader')
-        # TODO evand 2009-07-23: Zero out the MBR.  Check to see if the first
-        # 446 bytes are all NULs, and if not, ask the user if they want to wipe
-        # it.  Confirm with a USB disk that never has had an OS installed to
-        # it.
         self.progress_message(_('Installing the bootloader...'))
+        message = _('Failed to install the bootloader.')
         if sys.platform == 'win32':
+            # TODO evand 2009-07-23: Zero out the MBR.  Check to see if the
+            # first 446 bytes are all NULs, and if not, ask the user if they
+            # want to wipe it.  Confirm with a USB disk that never has had an
+            # OS installed to it.
             opts = '-fma'
             dev = str(os.path.splitdrive(self.target)[0])
+            try:
+                popen(['syslinux', opts, dev])
+            except (USBCreatorProcessException, IOError):
+                self._failure(message)
         else:
-            # FIXME evand 2009-07-31: This needs PolicyKit as you cannot run
-            # access a block device with syslinux as a regular user.
-            opts = '-f'
-            dev = self.device
-        try:
-            popen(['syslinux', opts, dev])
-        except (USBCreatorProcessException, IOError), e:
-            message = _('Failed to install the bootloader.')
-            if e:
-                message = message + '\n\n' + str(e)
-            self._failure(message)
+            import dbus
+            try:
+                bus = dbus.SystemBus()
+                obj = bus.get_object('com.ubuntu.USBCreator',
+                                     '/com/ubuntu/USBCreator')
+                obj.InstallBootloader(self.device,
+                                      dbus_interface='com.ubuntu.USBCreator',
+                                      timeout=MAX_DBUS_TIMEOUT)
+            except dbus.DBusException:
+                self._failure(message)
         self.check()
 
     def mangle_syslinux(self):
@@ -306,10 +310,21 @@
                'bs=1M']
         if sys.platform == 'win32':
             cmd.append('--size')
-        try:
-            popen(cmd)
-        except USBCreatorProcessException:
-            self._failure(failure_msg)
+            try:
+                popen(cmd)
+            except USBCreatorProcessException:
+                self._failure(failure_msg)
+        else:
+            import dbus
+            try:
+                bus = dbus.SystemBus()
+                obj = bus.get_object('com.ubuntu.USBCreator',
+                                     '/com/ubuntu/USBCreator')
+                obj.Image(self.source, self.device,
+                          dbus_interface='com.ubuntu.USBCreator',
+                          timeout=MAX_DBUS_TIMEOUT)
+            except dbus.DBusException:
+                self._failure(failure_msg)
 
     def cdimage_install(self):
         # Build.

=== modified file 'usbcreator/misc.py'
--- usbcreator/misc.py	2009-09-02 13:35:23 +0000
+++ usbcreator/misc.py	2009-09-16 15:07:07 +0000
@@ -18,6 +18,7 @@
 import os, sys
 import subprocess
 import tempfile
+from IN import INT_MAX
 
 # The minimum size, in megabytes a persistence file can be.
 MIN_PERSISTENCE = 128
@@ -28,6 +29,7 @@
               # disk image.
 ) = range(4)
 SOURCE_CD, SOURCE_ISO, SOURCE_IMG = range(3)
+MAX_DBUS_TIMEOUT = INT_MAX / 1000.0
 
 def setup_logging():
     log = logging.getLogger('')

